/**
 * $Id: mSQL.h,v 1.1 1997/07/09 18:59:55 k Exp $
 * 
 * (c)1995-1996 Alligator Descartes <descarte@hermetica.com>
 */

#include <DBIXS.h>		/* installed by the DBI module	*/

/** Installed during the installation of mSQL itself */
#include <msql.h>

/** read in our implementation details */
#include "dbdimp.h"
